const { PrismaClient } = require("@prisma/client");
const prisma = new PrismaClient();

async function createCategory(req, res) {
  const { name, slug, description, image } = req.body;

  try {
    const existingCategory = await prisma.category.findUnique({
      where: { slug },
    });
    if (existingCategory) {
      return res.status(400).json({ message: "Category slug already exists." });
    }

    const newCategory = await prisma.category.create({
      data: { name, slug, description, image },
    });

    res.status(201).json(newCategory);
  } catch (error) {
    console.error("Error creating category:", error);
    res
      .status(500)
      .json({ message: "Error creating category", error: error.message });
  }
}

async function getCategories(req, res) {
  try {
    const categories = await prisma.category.findMany({
      include: { products: true },
    });
    res.json(categories);
  } catch (error) {
    console.error("Error getting categories:", error);
    res
      .status(500)
      .json({ message: "Error getting categories", error: error.message });
  }
}

async function getCategoriesbyId(req, res) {
  const { id } = req.params;
  try {
    const category = await prisma.Category.findUnique({
      where: { id },
      include: {
        products: {
          include: {
            product: true,
          },
        },
      },
    });
    res.status(200).json(category);
  } catch (err) {
    console.error("Error fetching category with products:", err);
    res.status(500).json({ message: "Error fetching category with products" });
  }
}

async function putCategory(req, res) {
  const { id } = req.params;
  const { name, slug, description, image } = req.body;

  try {
    const category = await prisma.category.findUnique({ where: { id } });

    if (!category) {
      return res.status(404).json({ message: "Category not found." });
    }

    const updatedCategory = await prisma.category.update({
      where: { id },
      data: { name, slug, description, image },
    });

    res.status(200).json(updatedCategory);
  } catch (error) {
    console.error("Error updating category:", error);
    res
      .status(500)
      .json({ message: "Error updating category", error: error.message });
  }
}

async function deleteCategory(req, res) {
  const { id } = req.params;

  try {
    const category = await prisma.category.findUnique({ where: { id } });

    if (!category) {
      return res.status(404).json({ message: "Category not found." });
    }

    const delCategory = await prisma.category.delete({ where: { id } });
    res.json({ message: "Category deleted successfully", delCategory });
  } catch (error) {
    console.error("Error deleting category:", error);
    res
      .status(500)
      .json({ message: "Error deleting category", error: error.message });
  }
}

module.exports = {
  createCategory,
  getCategories,
  putCategory,
  deleteCategory,
  getCategoriesbyId,
};
