const { PrismaClient } = require("@prisma/client");
const prisma = new PrismaClient();

async function createProduct(req, res) {
  const { name, slug, description, image, categoryIDs } = req.body;

  try {
    const categoryIdsArray = Array.isArray(categoryIDs)
      ? categoryIDs
      : [categoryIDs];

    const newProduct = await prisma.Product.create({
      data: {
        name,
        slug,
        description,
        image,
        categories: {
          create: categoryIdsArray.map((categoryId) => ({
            category: {
              connect: { id: categoryId },
            },
          })),
        },
      },
      include: {
        categories: {
          include: {
            category: true,
          },
        },
      },
    });

    const transformedProduct = {
      ...newProduct,
      categories: newProduct.categories.map((pc) => pc.category),
    };

    res.status(201).json(transformedProduct);
  } catch (err) {
    console.error("Error creating product:", err);
    res.status(500).json({
      message: "Error creating product",
      error: err.message,
    });
  }
}

async function getProducts(req, res) {
  try {
    const products = await prisma.Product.findMany({
      include: {
        categories: {
          include: {
            category: true,
          },
        },
      },
    });
    res.json(products);
  } catch (error) {
    console.error("Error getting products:", error);
    res
      .status(500)
      .json({ message: "Error getting products", error: error.message });
  }
}

async function getProductById(req, res) {
  const { id } = req.params;

  try {
    const product = await prisma.Product.findUnique({
      where: { id },
      include: {
        categories: {
          include: {
            category: true,
          },
        },
      },
    });

    if (!product) {
      return res.status(404).json({ message: "Product not found" });
    }

    const transformedProduct = {
      ...product,
      categories: product.categories.map((pc) => pc.category),
    };

    res.status(200).json(transformedProduct);
  } catch (err) {
    console.error("Error fetching product:", err);
    res.status(500).json({
      message: "Error fetching product",
      error: err.message,
    });
  }
}

async function putProduct(req, res) {
  const { id } = req.params;
  const { name, slug, description, image, categorySlug } = req.body;

  try {
    const product = await prisma.Product.findUnique({ where: { id } });

    if (!product) {
      return res.status(404).json({ message: "Product not found." });
    }

    if (categorySlug) {
      const category = await prisma.category.findUnique({
        where: { slug: categorySlug },
      });
      if (!category) {
        return res
          .status(400)
          .json({ message: "Invalid categorySlug provided." });
      }
    }

    const updatedProduct = await prisma.Product.update({
      where: { id },
      data: { name, slug, description, image, categorySlug },
    });

    res.status(200).json(updatedProduct);
  } catch (error) {
    console.error("Error updating product:", error);
    res
      .status(500)
      .json({ message: "Error updating product", error: error.message });
  }
}

// Delete product
async function deleteProduct(req, res) {
  const { id } = req.params;

  try {
    const product = await prisma.Product.findUnique({ where: { id } });

    if (!product) {
      return res.status(404).json({ message: "Product not found." });
    }

    const delProduct = await prisma.Product.delete({ where: { id } });
    res.json({ message: "Product deleted successfully", delProduct });
  } catch (error) {
    console.error("Error deleting product:", error);
    res
      .status(500)
      .json({ message: "Error deleting product", error: error.message });
  }
}

module.exports = {
  createProduct,
  getProducts,
  putProduct,
  deleteProduct,
  getProductById,
};
